import { normalizePath, mergeConfig } from 'vite';
import { readFileSync } from 'node:fs';

function parseJson(json) {
  const result = {};
  function recurse(obj, currentKey) {
    for (const key in obj) {
      const value = obj[key];
      const newKey = currentKey ? currentKey + "-" + key : key;
      if (typeof value === "object" && value !== null && !Array.isArray(value)) {
        recurse(value, newKey);
      } else {
        result[newKey] = value;
      }
    }
  }
  recurse(json, "");
  return result;
}
function getPrefix(type) {
  switch (type) {
    case "css":
      return "--";
    case "less":
      return "@";
    case "stylus":
      return "";
    default:
      return "$";
  }
}
function getConnector(type) {
  if (type === "stylus") {
    return "= ";
  } else {
    return ": ";
  }
}
function getEOL(type) {
  switch (type) {
    case "sass":
    case "stylus":
      return "\n";
    default:
      return ";\n";
  }
}
function getVariableGenerator(lang) {
  const prefix = getPrefix(lang);
  const connector = getConnector(lang);
  const eol = getEOL(lang);
  return function(key, value) {
    return `${prefix}${key}${connector}${value}${eol}`;
  };
}
function convertObjToCssVar(obj, option) {
  const outputLang = option.style === "css" ? "css" : option.lang;
  const variiableGenerator = getVariableGenerator(outputLang);
  const data = Object.entries(obj).reduce((result, [key, value]) => result += variiableGenerator(key, value), "");
  return data;
}

const plugin = (option) => {
  let jsonFile;
  if (typeof option.file === "string") {
    const path = normalizePath(option.file);
    const fileContent = JSON.parse(readFileSync(path).toString());
    jsonFile = fileContent;
  } else {
    jsonFile = option.file;
  }
  const rawData = parseJson(jsonFile);
  const result = convertObjToCssVar(rawData, option);
  const virtualModuleId = "virtual:css-vars.css";
  const resolvedVirtualModuleId = "\0" + virtualModuleId;
  return {
    name: "vite-plugin-example",
    enforce: "pre",
    config(config) {
      if (option.style === "preprocessor") {
        const pluginConfig = {
          css: {
            preprocessorOptions: {
              [option.lang]: {
                additionalData: result
              }
            }
          }
        };
        return mergeConfig(pluginConfig, config);
      }
      return config;
    },
    resolveId(id) {
      if (id === virtualModuleId)
        return resolvedVirtualModuleId;
    },
    load(id) {
      if (id === resolvedVirtualModuleId && option.style === "css") {
        return {
          code: `
            :root{
              ${result}
            }
          `
        };
      }
    }
  };
};

export { plugin };
